@@ -27,63 +27,53 @@
 #include "WorkerInspectorController.h"
 
 #include "CommandLineAPIHost.h"
+#include "InspectorClient.h"
+#include "InspectorController.h"
 #include "InstrumentingAgents.h"
 #include "JSExecState.h"
+#include "Page.h"
+#include "ServiceWorkerAgent.h"
+#include "ServiceWorkerGlobalScope.h"
 #include "WebHeapAgent.h"
 #include "WebInjectedScriptHost.h"
 #include "WebInjectedScriptManager.h"
+#include "WorkerAuditAgent.h"
+#include "WorkerCanvasAgent.h"
 #include "WorkerConsoleAgent.h"
+#include "WorkerDOMDebuggerAgent.h"
+#include "WorkerDebugger.h"
 #include "WorkerDebuggerAgent.h"
-#include "WorkerGlobalScope.h"
 #include "WorkerNetworkAgent.h"
+#include "WorkerOrWorkletGlobalScope.h"
 #include "WorkerRuntimeAgent.h"
 #include "WorkerThread.h"
 #include "WorkerToPageFrontendChannel.h"
-#include <JavaScriptCore/InspectorAgent.h>
 #include <JavaScriptCore/InspectorAgentBase.h>
 #include <JavaScriptCore/InspectorBackendDispatcher.h>
 #include <JavaScriptCore/InspectorFrontendChannel.h>
 #include <JavaScriptCore/InspectorFrontendDispatchers.h>
 #include <JavaScriptCore/InspectorFrontendRouter.h>
 
-#if ENABLE(SERVICE_WORKER)
-#include "ServiceWorkerAgent.h"
-#include "ServiceWorkerGlobalScope.h"
-#endif
-
 namespace WebCore {
 
 using namespace JSC;
 using namespace Inspector;
 
-WorkerInspectorController::WorkerInspectorController(WorkerGlobalScope& workerGlobalScope)
+WorkerInspectorController::WorkerInspectorController(WorkerOrWorkletGlobalScope& globalScope)
     : m_instrumentingAgents(InstrumentingAgents::create(*this))
-    , m_injectedScriptManager(std::make_unique<WebInjectedScriptManager>(*this, WebInjectedScriptHost::create()))
+    , m_injectedScriptManager(makeUnique<WebInjectedScriptManager>(*this, WebInjectedScriptHost::create()))
     , m_frontendRouter(FrontendRouter::create())
     , m_backendDispatcher(BackendDispatcher::create(m_frontendRouter.copyRef()))
     , m_executionStopwatch(Stopwatch::create())
-    , m_scriptDebugServer(workerGlobalScope)
-    , m_workerGlobalScope(workerGlobalScope)
+    , m_globalScope(globalScope)
 {
-    ASSERT(workerGlobalScope.isContextThread());
+    ASSERT(globalScope.isContextThread());
 
     auto workerContext = workerAgentContext();
 
-    auto inspectorAgent = std::make_unique<InspectorAgent>(workerContext);
-    auto heapAgent = std::make_unique<WebHeapAgent>(workerContext);
-    auto consoleAgent = std::make_unique<WorkerConsoleAgent>(workerContext, heapAgent.get());
-
-    m_instrumentingAgents->setInspectorAgent(inspectorAgent.get());
+    auto consoleAgent = makeUnique<WorkerConsoleAgent>(workerContext);
     m_instrumentingAgents->setWebConsoleAgent(consoleAgent.get());
-
-    m_agents.append(WTFMove(inspectorAgent));
-    m_agents.append(std::make_unique<WorkerRuntimeAgent>(workerContext));
-    m_agents.append(std::make_unique<WorkerDebuggerAgent>(workerContext));
     m_agents.append(WTFMove(consoleAgent));
-    m_agents.append(WTFMove(heapAgent));
-
-    if (CommandLineAPIHost* commandLineAPIHost = m_injectedScriptManager->commandLineAPIHost())
-        commandLineAPIHost->init(m_instrumentingAgents->inspectorAgent(), m_instrumentingAgents->webConsoleAgent(), nullptr, nullptr, nullptr);
 }
 
 WorkerInspectorController::~WorkerInspectorController()
@@ -101,6 +91,8 @@ void WorkerInspectorController::workerTerminating()
     disconnectFrontend(Inspector::DisconnectReason::InspectedTargetDestroyed);
 
     m_agents.discardValues();
+
+    m_debugger = nullptr;
 }
 
 void WorkerInspectorController::connectFrontend()
@@ -117,9 +109,11 @@ void WorkerInspectorController::connectFrontend()
     m_executionStopwatch->reset();
     m_executionStopwatch->start();
 
-    m_forwardingChannel = std::make_unique<WorkerToPageFrontendChannel>(m_workerGlobalScope);
-    m_frontendRouter->connectFrontend(m_forwardingChannel.get());
+    m_forwardingChannel = makeUnique<WorkerToPageFrontendChannel>(m_globalScope);
+    m_frontendRouter->connectFrontend(*m_forwardingChannel.get());
     m_agents.didCreateFrontendAndBackend(&m_frontendRouter.get(), &m_backendDispatcher.get());
+
+    updateServiceWorkerPageFrontendCount();
 }
 
 void WorkerInspectorController::disconnectFrontend(Inspector::DisconnectReason reason)
@@ -134,8 +128,30 @@ void WorkerInspectorController::disconnectFrontend(Inspector::DisconnectReason r
     });
 
     m_agents.willDestroyFrontendAndBackend(reason);
-    m_frontendRouter->disconnectFrontend(m_forwardingChannel.get());
+    m_frontendRouter->disconnectFrontend(*m_forwardingChannel.get());
     m_forwardingChannel = nullptr;
+
+    updateServiceWorkerPageFrontendCount();
+}
+
+void WorkerInspectorController::updateServiceWorkerPageFrontendCount()
+{
+    if (!is<ServiceWorkerGlobalScope>(m_globalScope))
+        return;
+
+    auto serviceWorkerPage = downcast<ServiceWorkerGlobalScope>(m_globalScope).serviceWorkerPage();
+    if (!serviceWorkerPage)
+        return;
+
+    ASSERT(isMainThread());
+
+    // When a service worker is loaded in a Page, we need to report its inspector frontend count
+    // up to the page's inspectorController so the client knows about it.
+    auto inspectorClient = serviceWorkerPage->inspectorController().inspectorClient();
+    if (!inspectorClient)
+        return;
+
+    inspectorClient->frontendCountChanged(m_frontendRouter->frontendCount());
 }
 
 void WorkerInspectorController::dispatchMessageFromFrontend(const String& message)
@@ -159,7 +175,7 @@ WorkerAgentContext WorkerInspectorController::workerAgentContext()
 
     WorkerAgentContext workerContext = {
         webContext,
-        m_workerGlobalScope,
+        m_globalScope,
     };
 
     return workerContext;
@@ -172,13 +188,31 @@ void WorkerInspectorController::createLazyAgents()
 
     m_didCreateLazyAgents = true;
 
-#if ENABLE(SERVICE_WORKER)
+    m_debugger = makeUnique<WorkerDebugger>(m_globalScope);
+
+    m_injectedScriptManager->connect();
+
     auto workerContext = workerAgentContext();
-    if (is<ServiceWorkerGlobalScope>(m_workerGlobalScope)) {
-        m_agents.append(std::make_unique<ServiceWorkerAgent>(workerContext));
-        m_agents.append(std::make_unique<WorkerNetworkAgent>(workerContext));
+
+    m_agents.append(makeUnique<WorkerRuntimeAgent>(workerContext));
+
+    if (is<ServiceWorkerGlobalScope>(m_globalScope)) {
+        m_agents.append(makeUnique<ServiceWorkerAgent>(workerContext));
+        m_agents.append(makeUnique<WorkerNetworkAgent>(workerContext));
     }
-#endif
+
+    m_agents.append(makeUnique<WebHeapAgent>(workerContext));
+
+    auto debuggerAgent = makeUnique<WorkerDebuggerAgent>(workerContext);
+    auto debuggerAgentPtr = debuggerAgent.get();
+    m_agents.append(WTFMove(debuggerAgent));
+
+    m_agents.append(makeUnique<WorkerDOMDebuggerAgent>(workerContext, debuggerAgentPtr));
+    m_agents.append(makeUnique<WorkerAuditAgent>(workerContext));
+    m_agents.append(makeUnique<WorkerCanvasAgent>(workerContext));
+
+    if (auto& commandLineAPIHost = m_injectedScriptManager->commandLineAPIHost())
+        commandLineAPIHost->init(m_instrumentingAgents.copyRef());
 }
 
 InspectorFunctionCallHandler WorkerInspectorController::functionCallHandler() const
@@ -191,9 +225,20 @@ InspectorEvaluateHandler WorkerInspectorController::evaluateHandler() const
     return WebCore::evaluateHandlerFromAnyThread;
 }
 
+Stopwatch& WorkerInspectorController::executionStopwatch() const
+{
+    return m_executionStopwatch;
+}
+
+JSC::Debugger* WorkerInspectorController::debugger()
+{
+    ASSERT_IMPLIES(m_didCreateLazyAgents, m_debugger);
+    return m_debugger.get();
+}
+
 VM& WorkerInspectorController::vm()
 {
-    return m_workerGlobalScope.vm();
+    return m_globalScope.vm();
 }
 
 } // namespace WebCore
